/*
 qzlog パッケージは、ログ・ツールとして、アンセーフなC言語で記述されたDLLに内包された関数を呼び出します。

 1. 開始時には必ず LogInitで初期化し、内部メモリをアロケートさせ、ファイルハンドルをオープンします。
 2. 終了前には必ず LogTermで、内部メモリをフリーし、ファイルハンドルをクローズしなければなりません。
 3. LogInitが成功すれば、一行ログのlog系関数と、ダンプ関数、ビット関数、1文字出力関数が利用可能となります。
   Copywrite(C) QuiZ Lab. Cool Bar! 2021.01/20 netbsdmania@gmail.com
   本ファイルは、改変もご自由に行ってください。
   ご自由によりgoらしい書き方に改変してご利用ください。
   改変しても、オリジナルのCopyrightは残してください。あとはご自由に。

  ■注意事項 このログ・ライブラリは、シングルスレッドでの学習用途を想定しております。
  マルチスレッドサーバなどでのご利用はどうなるかわかりません。今後 非同期IO/IOCP完了ポートによる
  マルチスレッド用のリリースも考えておりますが、それは業務用途や、WWWサーバなどでのデバッグ
  用になるかと思います。
　version 0.5.1.2
*/
package qzlogs

import (
	"syscall"
	"unsafe"
)

var (
	// 以下の行のパス位置は、このパッケージをインストールした場所に依存します。正確に書き換えてご利用ください。
	// また losplgo_x64.dll, logsplgo_x64.libも必ず同じパス位置に配置してください。
	qzlogsU8       = syscall.NewLazyDLL("D:\\PrgFiles\\go\\src\\qzlogs\\logsplgo_x64.dll")
	logInitRaw     = qzlogsU8.NewProc("logInit")
	logTermRaw     = qzlogsU8.NewProc("logTerm")
	logRaw         = qzlogsU8.NewProc("_log")
	logDebugRaw    = qzlogsU8.NewProc("log_debug")
	logInfoRaw     = qzlogsU8.NewProc("log_info")
	logWarningRaw  = qzlogsU8.NewProc("log_warning")
	logErrorRaw    = qzlogsU8.NewProc("log_error")
	logCriticalRaw = qzlogsU8.NewProc("log_critical")

	logDumpUTF8Raw                     = qzlogsU8.NewProc("logDumpUTF8")
	logPutRaw                          = qzlogsU8.NewProc("logPut")
	logPutHexRaw                       = qzlogsU8.NewProc("logPutHex")
	makeBitStringUINT32Raw             = qzlogsU8.NewProc("makeBitStringUINT32")
	makeBitStringUSHORT16Raw           = qzlogsU8.NewProc("makeBitStringUSHORT16")
	makeBitStringUnicodeScalarValueRaw = qzlogsU8.NewProc("makeBitStringUnicodeScalarValue")
	msgBoxRaw                          = qzlogsU8.NewProc("msgBox")
	msgBoxOkCancelRaw                  = qzlogsU8.NewProc("msgBoxOkCancel")
	getLastErrRaw                      = qzlogsU8.NewProc("GetLastErr")
	profileTimeStartRaw                = qzlogsU8.NewProc("profileTimeStart")
	profileTimeStopRaw                 = qzlogsU8.NewProc("profileTimeStop")
	w32ResetErrCodeRaw                 = qzlogsU8.NewProc("w32_reset_errCode")
	w32ErrLogRaw                       = qzlogsU8.NewProc("w32_err_log")
)

// LogInit  Unsafe dll Call and File Handle Init
func LogInit(path string, prefix string, writeFlg int, newLineCode int, logSleepSw int, logLevel int, utf8Bom int) (retval uint32, err error) {
	ret, _, _ :=
		syscall.Syscall9(logInitRaw.Addr(), 7,
			uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(path))),
			uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(prefix))),
			uintptr(writeFlg),
			uintptr(newLineCode),
			uintptr(logSleepSw),
			uintptr(logLevel),
			uintptr(utf8Bom),
			0, 0)

	return uint32(ret), err
}

// LogTerm Unsafe dll Allocated memory and FIle Handle Free & Close
func LogTerm() {
	logTermRaw.Call()
}

// Log Write the Open Log text One Line Logging.
func Log(data string) {
	logRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// LogDebug  Write the Open Log text One Line Logging. LogInit No.6 Param 1-5  if 5 upper then Write!
func LogDebug(data string) {
	logDebugRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// LogInfo  Write the Open Log text One Line Logging. LogInit No.6 Param 1-5  if 4 upper then Write!
func LogInfo(data string) {
	logInfoRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// LogWarning  Write the Open Log text One Line Logging. LogInit No.6 Param 1-5  if 3 upper then Write!
func LogWarning(data string) {
	logWarningRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// LogError  Write the Open Log text One Line Logging. LogInit No.6 Param 1-5  if 2 upper then Write!
func LogError(data string) {
	logErrorRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// LogCritical  Write the Open Log text One Line Logging. LogInit No.6 Param 1-5  if 1 upper then Write!
func LogCritical(data string) {
	logCriticalRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// LogDumpUTF8 Write the Open log text Dump Data Write. prefix is a Identify chars.
func LogDumpUTF8(data string, size int, prefix string) {
	logDumpUTF8Raw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))),
		uintptr(size),
		uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(prefix))))
}

// LogPut One Char Put. loop cnt size. header int 0... No Header Write. 1...Header Write.
func LogPut(data string, cnt int, header int, newLineCnt int) {
	logPutRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))),
		uintptr(cnt),
		uintptr(header),
		uintptr(newLineCnt))
}

// LogPutHex One Char Put.(0xXX)Style. loop cnt size. header int 0... No Header Write. 1...Header Write.
func LogPutHex(data string, cnt int, header int, newLineCnt int) {
	logPutHexRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))),
		uintptr(cnt),
		uintptr(header),
		uintptr(newLineCnt))
}

// MakeBitStringUINT32 x is the Bit Print data (unsigned 4bytes)  data string is a User define strings.
func MakeBitStringUINT32(x uint32, data string) {
	makeBitStringUINT32Raw.Call(uintptr(x), uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// MakeBitStringUSHORT16 x is the Bit Print data (unsigned 2bytes)  data string is a User define strings.
func MakeBitStringUSHORT16(x uint16, prefix string) {
	makeBitStringUSHORT16Raw.Call(uintptr(x), uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(prefix))))
}

// MakeBitStringUnicodeScalarValue x is the Bit Print data (UNICODE Scalar unsigned 4bytes)  data string is a User define strings.
func MakeBitStringUnicodeScalarValue(x uint32, data string) {
	makeBitStringUnicodeScalarValueRaw.Call(uintptr(x), uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))))
}

// MsgBox  data string is a Inner Dialog Strings, ttl is a Tittle Bar Strings.
func MsgBox(data string, ttl string) {
	msgBoxRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))),
		uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(ttl))))
}

// MsgBoxOkCancel  data string is a Inner Dialog Strings, ttl is a Tittle Bar Strings.
func MsgBoxOkCancel(data string, ttl string) (retval uint32, err error) {

	ret, _, err := syscall.Syscall(msgBoxOkCancelRaw.Addr(), 2,
		uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(data))),
		uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(ttl))),
		0)

	return uint32(ret), err
}

// ProfileTimeStart マイクロ秒計測開始
func ProfileTimeStart(prefix string) {
	_, _, _ = syscall.Syscall(profileTimeStartRaw.Addr(), 1,
		uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(prefix))), 0, 0)
}

// ProfileTimeStop マイクロ秒計測終了
func ProfileTimeStop() {
	_, _, _ = syscall.Syscall(profileTimeStopRaw.Addr(), 0, 0, 0, 0)
}

// W32ResetErrCode  Win32 Error Code Reset NO_ERROR set.
func W32ResetErrCode() {
	w32ResetErrCodeRaw.Call()
}

// W32ErrLog Writing with Win32 Error Code & Message(JP)
func W32ErrLog(fileFnc string, line int) {

	w32ErrLogRaw.Call(uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(fileFnc))),
		uintptr(line))
}

// End of qzlogsUTF8.go
